@@ -1,90 +1,13 @@
-import requests
 import json
 import pandas as pd
-import pytz
-from datetime import datetime, timedelta
 from dateutil.parser import parse
-from progress.bar import Bar
 
-tz = pytz.timezone('Europe/Madrid')
-
-API_URL = "https://flowmaps.life.bsc.es/api"
-
-
-def date_rfc1123(dt):
-    """Return a string representation of a date according to RFC 1123
-    (HTTP/1.1).
-
-    The supplied date must be in UTC.
-
-    """
-    weekday = ["Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"][dt.weekday()]
-    month = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep",
-             "Oct", "Nov", "Dec"][dt.month - 1]
-    return "%s, %02d %s %04d %02d:%02d:%02d GMT" % (weekday, dt.day, month,
-        dt.year, dt.hour, dt.minute, dt.second)
-
-
-def parse_date(date_str):
-    return tz.localize(datetime.strptime(date_str, "%Y-%m-%d")) 
-
-
-def clean_docs(docs):
-    skip_fields = ['_links', '_created', '_updated', 'href', '_etag']
-    for doc in docs:
-        for field in skip_fields:
-            if field in doc:
-                del doc[field]
-    return docs
-
-
-def fetch_first(collection, query, projection={}):
-    base_url = API_URL
-    url = f"{base_url}/{collection}"
-    params = {'where': json.dumps(query), 'max_results': 1, 'projection': json.dumps(projection)}
-    # print(f"API url: {base_url}/{collection}?where={params['where']}&max_results={params['max_results']}&projection={params['projection']}")
-    response = requests.get(url, params=params).json()
-    if not response or not response.get('_items'):
-        return None
-    return response['_items'][0]
-
-
-def fetch_all_pages(collection, query, batch_size=1000, projection={}, sort=None, progress=True, print_url=False):
-    base_url = API_URL
-    url = f"{base_url}/{collection}"
-    params = {'where': json.dumps(query), 'max_results': batch_size, 'projection': json.dumps(projection)}
-    if sort:
-        params['sort'] = sort
-    data = []
-    if print_url:
-        print(f"API request: {base_url}/{collection}?where={params['where']}")
-    response = requests.get(url, params=params).json() # get first page
-    data.extend(response['_items'])
-    if '_links' not in response:
-        return data
-    num_docs = response['_meta']['total']
-    if num_docs <= 0:
-        return data
-    num_pages = int(num_docs/batch_size)
-    if progress: bar = Bar('Dowloading documents', max=num_docs)
-    while 'next' in response['_links']:
-        if progress: bar.goto(len(data))
-        url = f"{base_url}/{response['_links']['next']['href']}"
-        response = requests.get(url).json()
-        data.extend(response['_items'])
-    if progress: bar.goto(len(data))
-    if progress: bar.finish()
-    return data
+from .utils import fetch_first, fetch_all_pages
+from .data import layer, covid19, dataset, daily_mobility_matrix, population, zone_movements
 
 
 def list_layers():
     print('Listing layers:')
-    # filters = {
-    #     'collection': 'layers', 
-    #     'field': 'layer', 
-    # }
-    # data = fetch_all_pages('distinct', filters)
-    # print("\n".join(data))
     filters = {
         'storedIn': 'layers', 
     }
@@ -117,15 +40,9 @@ def describe_layer(layer, provenance=False, plot=False):
 def download_layer(layer, output_file, plot=False, no_save=False):
     if output_file is None:
         output_file = layer+'.geojson'
+
     print(f'Dowloading layer {layer}')
-    filters = {
-        'layer': layer
-    }
-    data = fetch_all_pages('layers', filters, print_url=True)
-    featureCollection = {
-        "type": "FeatureCollection",
-        "features": [doc['feat'] for doc in data],
-    }
+    featureCollection = layer(layer, print_url=True)
 
     if not no_save:
         print(f'Saving layer to file: {output_file}')
@@ -145,15 +62,8 @@ def download_layer(layer, output_file, plot=False, no_save=False):
             print(f"\n\n  WARN: {e}. \n  To use the --plot option you need to install the following packages: geopandas, descartes, matplotlib. \n  For example, use:  pip install geopandas descartes matplotlib")
 
 
-def list_health():
+def list_covid19():
     print('Listing consolidated ev:')
-    # filters = {
-    #     'collection': 'layers.data.consolidated', 
-    #     'field': 'ev',
-    #     'query': {'type': 'covid19'},
-    # }
-    # data = fetch_all_pages('distinct', filters)
-    # print("\n".join(data))
     filters = {
         'storedIn': 'layers.data.consolidated',
         'keywords.type': 'covid19', 
@@ -163,7 +73,7 @@ def list_health():
         print(f"{doc['keywords']['ev']}\n\tDescription: {doc.get('processedFrom', [{}])[0].get('keywords', {}).get('evDesc')}\n\tNumber of entries: {doc['numEntries']}\n\tlayer: {doc['keywords']['layer']}\n")
 
 
-def describe_health(ev, provenance=False):
+def describe_covid19(ev, provenance=False):
     print(f'Describing consolidated ev={ev}')
     filters = {
         'storedIn': 'layers.data.consolidated',
@@ -182,26 +92,21 @@ def describe_health(ev, provenance=False):
     }
     data = fetch_all_pages('distinct', filters, progress=False)
     print(f"Available dates: min={min(data)}, max={max(data)}")
-    if provenance:
-        print(f"Full provenance: {json.dumps(prov, indent=4)}")
 
     example = fetch_first('layers.data.consolidated', {'type': 'covid19', 'ev': ev})
     print("Example document:\n"+json.dumps(example, indent=4))
 
+    if provenance:
+        print(f"Full provenance: {json.dumps(prov, indent=4)}")
 
-def download_health(ev, output_file, output_format='csv', start_date=None, end_date=None):
+
+def download_covid19(ev, output_file, output_format='csv', start_date=None, end_date=None):
     print(f'Dowloading consolidated health data for ev={ev}')
     filters = {
         'ev': ev,
         'type': 'covid19',
     }
-    if start_date and end_date:
-        filters['date'] = {'$gte': start_date, '$lte': end_date}
-    elif start_date:
-        filters['date'] = {'$gte': start_date}
-    elif end_date:
-        filters['date'] = {'$lte': end_date}
-    data = fetch_all_pages('layers.data.consolidated', filters, print_url=True)
+    data = covid19(ev, start_date=start_date, end_date=end_date, fmt='records', print_url=True)
     if output_format == 'csv':
         df = pd.DataFrame(data)
         df.to_csv(output_file, index=False)
@@ -216,13 +121,6 @@ def download_health(ev, output_file, output_format='csv', start_date=None, end_d
 
 def list_data():
     print('Listing ev:')
-    # filters = {
-    #     'collection': 'layers.data', 
-    #     'field': 'ev',
-    #     'query': {},
-    # }
-    # data = fetch_all_pages('distinct', filters)
-    # print("\n".join(data))
     filters = {
         'storedIn': 'layers.data',
     }
@@ -231,8 +129,6 @@ def list_data():
         print(f"{doc['keywords']['ev']}\n\tDescription: {doc['keywords'].get('evDesc', '')}\n\tlayer: {doc['keywords'].get('layer')}\n")
 
 
-
-
 def describe_data(ev, provenance=False):
     print(f'Describing ev={ev}')
     filters = {
@@ -262,16 +158,7 @@ def describe_data(ev, provenance=False):
 
 def download_data(ev, output_file, output_format='csv', start_date=None, end_date=None):
     print(f'Dowloading data for ev={ev}')
-    filters = {
-        'ev': ev,
-    }
-    if start_date and end_date:
-        filters['evstart'] = {'$gte': date_rfc1123(parse_date(start_date)), '$lte': date_rfc1123(parse_date(end_date) + timedelta(days=1))}
-    elif start_date:
-        filters['evstart'] = {'$gte': date_rfc1123(parse_date(start_date))}
-    elif end_date:
-        filters['evstart'] = {'$lte': date_rfc1123(parse_date(end_date) + timedelta(days=1))}
-    data = fetch_all_pages('layers.data', filters, print_url=True)
+    data = dataset(ev, start_date=start_date, end_date=end_date, fmt='records', print_url=True)
     if output_format == 'csv':
         df = pd.DataFrame(data)
         df.to_csv(output_file, index=False)
@@ -339,16 +226,7 @@ def describe_daily_mobility_matrix(provenance=False):
 
 def download_daily_mobility_matrix(source_layer, target_layer, date, output_file, output_format='csv', source=None, target=None):
     print(f'Dowloading mobility matrix for source_layer={source_layer} target_layer={target_layer} date={date}')
-    filters = {
-        'date': date,
-        'source_layer': source_layer,
-        'target_layer': target_layer,
-    }
-    if source:
-        filters['source'] = source
-    if target:
-        filters['target'] = target
-    data = fetch_all_pages('mitma_mov.daily_mobility_matrix', filters, print_url=True)
+    data = daily_mobility_matrix(source_layer, target_layer, date, fmt='records', print_url=True, source=source, target=target)
     if output_format == 'csv':
         df = pd.DataFrame(data)
         df.to_csv(output_file, index=False)
@@ -401,17 +279,8 @@ def describe_population(layer, provenance=False):
 
 def download_population(layer, output_file, output_format='csv', start_date=None, end_date=None):
     print(f'Dowloading population for layer={layer}')
-    filters = {
-        'layer': layer,
-        'type': 'population',
-    }
-    if start_date and end_date:
-        filters['date'] = {'$gte': start_date, '$lte': end_date}
-    elif start_date:
-        filters['date'] = {'$gte': start_date}
-    elif end_date:
-        filters['date'] = {'$lte': end_date}
-    data = fetch_all_pages('layers.data.consolidated', filters, print_url=True)
+    data = population(layer, start_date=start_date, end_date=end_date, fmt='records', print_url=True)
+    
     if output_format == 'csv':
         df = pd.DataFrame(data)
         df.to_csv(output_file, index=False)
@@ -467,17 +336,8 @@ def describe_zone_movements(provenance=False):
 
 def download_zone_movements(layer, output_file, output_format='csv', start_date=None, end_date=None):
     print(f'Dowloading population for layer={layer}')
-    filters = {
-        'layer': layer,
-        'type': 'zone_movements',
-    }
-    if start_date and end_date:
-        filters['date'] = {'$gte': start_date, '$lte': end_date}
-    elif start_date:
-        filters['date'] = {'$gte': start_date}
-    elif end_date:
-        filters['date'] = {'$lte': end_date}
-    data = fetch_all_pages('layers.data.consolidated', filters, print_url=True)
+    data = zone_movements(layer, start_date=start_date, end_date=end_date, fmt='records', print_url=True)
+    
     if output_format == 'csv':
         df = pd.DataFrame(data)
         df.to_csv(output_file, index=False)
